/***************************************************************************
 *            chatwindow.h
 *
 *  Tue Feb 15 04:08:09 2005
 *  Copyright  2005  User
 *  Email
 ****************************************************************************/

#ifndef _CHATWINDOW_H
#define _CHATWINDOW_H

#include <gtk/gtk.h>
#include "icons.h"
#include "systemcharset.h"
#include "basicwindow.h"
#include "imuserdaemon.h"
#include "processingani.h"
#include "requestdialog.h"
#include "gettext.h"
#include "userutilities.h"

enum
{
	CWVT_PROTO,
	CWVT_WAIT,
	CWVT_REMOTE,
	CWVT_OWNER,
	CWVT_IRC
};

enum
{
	CF_FGCOLOR 		= 1<<0,
	CF_BGCOLOR 		= 1<<1,
	CF_FAMILY		= 1<<2,
	CF_SIZE			= 1<<3,	
	CF_FACE			= 1<<4,
	CF_ALL			= 0xFF
};


// FIXME: dirty!
class chatWindow;


class chatWindowView
{
	public:
	
		GtkWidget	*textBox,
					*boxFrame,
					*parentBox;
	
		gchar	*alias,
				*userID,
				*charset;

		GString *currentLine,
				*history;	
	
		gchar	viewType;
	
		GtkTextTag 	*fontProperties;
		GtkTreeIter listIter;
	
		gboolean useUserFont,
				 useUserColors;
	
		chatWindow* parentWindow;
	
	
		chatWindowView(gchar* _alias, gchar* _userID, chatWindow* _parentWindow);
		virtual ~chatWindowView();
	
		virtual GtkWidget* createChatView();
		virtual void destroyChatView();
	
		virtual GtkWidget* createToolBar();	
	
		void updateStyle(chatEventInfo *ev);
		void createFontContext();
	
	protected:
		GtkWidget	*encodingsMenu;
	
		void		setSelectedEncodingsMenuItem(gchar* id);
	
		virtual void encodingChanged(charsetType* enc);
	
		
		static void cb_showSubmenu(GtkWidget* subMenu);
		static void cb_selectEncoding(GtkWidget* item, chatWindowView *cwv);	
};







class chatWindowLocalView: public chatWindowView
{
	public:
		GtkWidget	*ircButton;	
	
		chatWindowLocalView(chatWindow* _parentWindow);
		virtual ~chatWindowLocalView();	
	
		virtual GtkWidget* createToolBar();
		virtual GtkWidget* createChatView();
		void	updateFontInformation(guint toUpdate);
		void	clearTextBuffer();
	
	private:
		guint	currentEncoding;
	
		GtkWidget* createEncodingsMenu();
		GtkWidget* createColorMenus(gboolean createBG);
	
		void encodingChanged(charsetType* enc);
	
		static gint cb_sortFamilies(PangoFontFamily** a, PangoFontFamily** b);
	
		static void cb_selectColor(GtkWidget* item, chatWindowLocalView *cwv);
		static void cb_selectFont(GtkWidget* box, chatWindowLocalView *cwv);
		static void cb_selectSize(GtkWidget* box, chatWindowLocalView *cwv);
		static void cb_selectBold(GtkWidget* btn, chatWindowLocalView *cwv);
		static void cb_selectItalic(GtkWidget* btn, chatWindowLocalView *cwv);
		static void cb_selectUnderline(GtkWidget* btn, chatWindowLocalView *cwv);
		static void cb_selectStrikeout(GtkWidget* btn, chatWindowLocalView *cwv);
	
		static gboolean cb_keyPressed(GtkWidget* w, GdkEventKey* event, chatWindowLocalView *cwv);
		static void	cb_textInserted(GtkTextBuffer *textbuffer, GtkTextIter *arg1, gchar *arg2, gint arg3, chatWindowLocalView* cw);
};



class chatWindowRemoteView: public chatWindowView
{
	public:	
		chatWindowRemoteView(gchar* _alias, gchar* _userID, chatWindow* _parentWindow);
		virtual ~chatWindowRemoteView();
	
		GtkWidget* createChatView();
	
		void insertCharacter(gchar* character);
		void deleteCharacter();
	
		gchar* getCurrentLineAndClear();
	
		virtual GtkWidget* createToolBar();
	
		void	kickUser(gchar* alias, gulong uin);
		GtkMenu	*createUserMenu();
	
	private:
		GtkWidget	*fontButtonImg,
					*colButtonImg;
	
		GtkMenu		*userMenu;
	
	
	static void cb_setUseFont(chatWindowRemoteView *crv);
	static void cb_setUseColors(chatWindowRemoteView *crv);
	static void cb_kickUserButtonClicked(chatWindowRemoteView *crv);
	static void cb_menuItemClicked(GtkMenuItem* item, chatWindowRemoteView* crv);
};





class chatWindowIRCView: public chatWindowView
{
	public:
		GtkTextBuffer *textBuffer;	
		GtkTextTag	  *fontComment,
					  *fontAlias;
	
		chatWindowIRCView(chatWindow *_parentWindow);
		virtual ~chatWindowIRCView();
	
		virtual GtkWidget* createChatView();
	
		void insertLine(gchar* alias, gchar* line, GtkTextTag* font, gboolean isComment);
};




class chatWindow: public basicWindow
{
	public:
		GtkWidget		*mainChatBox,
						*chatVBox,
						*lastFreeCVPlace,
						*userList,
						*userListFrame;
	
		GtkListStore	*usersStore;
	
		GList				*chatViews;
		chatWindowLocalView	*ownerView;
		chatWindowIRCView 	*ircView;
	
		gboolean		isIRCStyle,
						isServer;
	
		chatWindow		*multipartyWindow;
	
		processingAni	*connectAni;
	
		requestDialog	*request;
	
//		remoteRequestDialog* remoteRequest;
//		localRequestDialog* localRequest;

		gint	requestID;
		
		chatWindow(gboolean _isServer);
		~chatWindow();
	
		gchar *getChatWindowTitle();
	
		void focusWindow();
		void createRequestDialog(chatEventInfo* event);
	
		void rebuildViewMode();
		
		void startChat();	
	
	
		static void cb_changeViewButtonClicked(chatWindow* chw);
		static void cb_chatModeSelectionClicked(chatWindow* chw);

	private:
		GtkWidget	*chatRoomList,
					*startNewChatButton,
					*joinExistingChatButton;	
	
		virtual GtkWidget* createWindowContent();
		virtual void	   destroyWindowContent();
		virtual void	   windowActivated();
		virtual void	   windowDeactivated();

		virtual gboolean eventCallback(gint command, gint lastResult, gpointer info);	
	
		GtkWidget* createButtonbar();
		GtkWidget* createUserList();
		GtkWidget* createSpecialWidget(gboolean canMultiparty);
		chatWindow* getMultipartyWindow();
		void addViewToList(chatWindowView* view);
		void removeViewFromList(chatWindowView* view);
	
		GtkWidget* createUserSelection();
	
		chatFontInfo* getFontInfo();
	
		void	   addChatView(chatWindowView* cwv);
	
		chatWindowView* findViewByID(gchar* id);
	
	
		// callbacks	
		static void cb_requestCallback(gint result, gpointer info, chatWindow* cw);
		static gboolean cb_userListClicked(GtkTreeView* uList, GdkEventButton* event, chatWindow* cw);		

};



#endif /* _CHATWINDOW_H */
