/***************************************************************************
 *            imownerdaemon.h
 *
 *  Mon Jan 31 02:45:40 2005
 *  Copyright  2005  User
 *  Email
 ****************************************************************************/

#ifndef _IMOWNERDAEMON_H
#define _IMOWNERDAEMON_H

#include <gtk/gtk.h>
#include <iostream>
#include "imuserdaemon.h"
#include "licq_log.h"

typedef struct redirectedEvent
{
	gchar	*newLicqID;
	gint	eventID;
};




typedef struct searchEventInfo: generalEventInfo
{
	gchar		*id,
				*alias,
				*firstName,
				*lastName,
				*email;
	
	gint		more;
	gboolean 	lastEntry;
	gchar		status,
				gender,
				age;
	gboolean 	requireAuthorization;
};


typedef struct panelEventInfo: basicEventInfo
{
	gchar		*name,
				*ip,
				*email;
};


typedef struct logInfo
{
	gchar	*message;
	gushort type;
};

//! the logging daemon
class IMLogDaemon: public CPluginLog, public IMSignalSource
{
	public:
		GList	*loggedInfo;		
	
		IMLogDaemon();
		~IMLogDaemon();
	
		void	clearLog();
		
	private:
		guint 		watchSource;
		GIOChannel	*channel;	
	
		static gboolean cb_pipeCallback(GIOChannel* source, GIOCondition condition, IMLogDaemon* self);	
};


class IMConnectionsManager
{
	public:
		IMConnectionsManager();
		~IMConnectionsManager();
	
		// Firewall 
		gboolean 	 getFirewallEnabled()	{return getLicqDaemon()->Firewall(); }
		gboolean 	 getFirewallDirect()	{return getLicqDaemon()->TCPEnabled(); }
		gushort	 	 getFirewallPortsFrom()	{return getLicqDaemon()->TCPPortsLow(); }
		gushort  	 getFirewallPortsTo()	{return getLicqDaemon()->TCPPortsHigh(); }
		
		// Proxy
		gboolean 	 getProxyEnabled()		{return getLicqDaemon()->ProxyEnabled(); }
		const gchar	 *getProxyServer()		{return getLicqDaemon()->ProxyHost(); }
		gushort  	 getProxyPort()			{return getLicqDaemon()->ProxyPort(); }
		gboolean 	 getProxyAuthEnabled()	{return getLicqDaemon()->ProxyAuthEnabled(); }
		const gchar	 *getProxyUsername()	{return getLicqDaemon()->ProxyLogin(); }
		const gchar	 *getProxyPassword()	{return getLicqDaemon()->ProxyPasswd(); }
		
		gboolean 	 getKeepConnection()	{return getLicqDaemon()->ReconnectAfterUinClash(); }
		
};



//! the random chat manager
class IMRandomChatManager: public IMEventManager
{
	public:
		IMRandomChatManager(IMUserInfo* _info);
		~IMRandomChatManager();
	
		void setChatGroup(guint searchNum);
		gint getChatGroup();
		void searchForChatPartner(guint searchNum);
	
		gboolean finishEvent(ICQEvent* event);
	
	private:
		gulong getIcqChatNumber(guint num);
};



//! a manager that handles all kinds of notifications
// (message events, panel, pager ...)
class IMNotificationManager: public IMEventManager
{
	public:
		IMNotificationManager(IMUserInfo* _info);
		~IMNotificationManager();
	
		protected:
			gpointer mergeEvent(CUserEvent* event);
};



//! the user search manager 
class IMUserSearchManager: public IMEventManager
{
	public:
		IMUserSearchManager(IMUserInfo* _info);
		~IMUserSearchManager();
	
		void searchWhitePages(gchar* firstName, gchar* lastName,
							  gchar* alias, gchar* email,
							  gushort minAge, gushort maxAge,
							  gchar gender, gchar language,
							  gchar* city, gchar* state, gushort country,
							  gchar* companyName, gchar *companyDep,
							  gchar* companyPos,
							  gchar* keyWord,
							  gboolean onlineOnly);
	
		void searchByID(gchar* id);
	
		void setRandomChatGroup(gulong group);
		gulong getRandomChatGroup();
		void searchRandomChatGroup(gulong group);
	
		gboolean finishEvent(ICQEvent* event);
	
	
	private:
		gulong randomChatGroup;
};



// the protocol daemon - manages protocol functions such as loading/unloading
class IMPluginDaemon
{
	public:
		gboolean isProtoPlugin,
				 isEnabled,
				 isLoaded;
		gchar	 *name,
				 *version,
			     *libraryName,
				 *licqName,
				 *description;
	
		gushort	 id;
		gulong	 pluginID,
				 functions;
	
		GList	 *owners;
	
	
	
		IMPluginDaemon(CProtoPlugin* licqProto);
		IMPluginDaemon(CPlugin* licqProto);
		IMPluginDaemon(gchar* _libraryName);
		~IMPluginDaemon();
	
		gboolean loadPlugin();
		gboolean unloadPlugin();
		gboolean registerUser(gchar* userName, gchar* password);
		gboolean verifyRegistration(gchar* verifyStr);
		gboolean addExistingUser(gchar* userName, gchar* password);
		
		gchar	 *getServerName();
		gushort	 getServerPort();
		void	 setServerName(gchar *name);
		void	 setServerPort(gushort port);
		
		void removeOwner(IMOwnerDaemon* toRemove);
		
		void evaluatePluginSignal(CICQSignal* signal);
		void evaluatePluginEvent(ICQEvent* event);
		
		void refreshProtoInfo(CProtoPlugin *plugin, gboolean refresh);
		void refreshPluginInfo(CPlugin* plugin, gboolean refresh);
		void refreshUnloadedInfo(gchar* lib, gboolean refresh);
};




// the owner daemon - contains every function that might be neccessary for
//					  protocol events
class IMOwnerDaemon: public IMUserDaemon
{
	public:	
		GList			*users,
						*ignoredUsers;
		IMPluginDaemon	*protocol;
		IMGroupManager	*groupMan;
		guint			oldStatus;
	
		IMOwnerDaemon(gchar* _ownerID, guint _pluginID);
		virtual ~IMOwnerDaemon();

		// dispatches owner events (and directs them further if neccessary)
		virtual void dispatchPendingEvents();	
	
		virtual void dispatchInformation(CICQSignal* signal);

		// load all contacts for this protocol
		void loadContacts();
		void loadIgnoredContacts();
	
		void clearIgnoredContacts();
	
		// set the status
		void setStatus(guint status);
	
		// sets the user invisible
		void setInvisible(gboolean setting);
				
		void evaluateLicqSignal(CICQSignal* signal);
		void evaluateLicqEvent(ICQEvent* event);
		IMUserDaemon *getLastEvent(basicEventInfo** toFill, gboolean favourSystemEvents);
	
		IMUserDaemon* findUserByID(gchar* id);	
		IMUserDaemon* findUserByConversationID(gulong cID);
		IMUserDaemon* findUserByEvent(ICQEvent* event);
		
		IMUserDaemon* addUserTemporally(gchar* idString);
		void		  addUserToList(gchar* idString, gboolean alertUser);
		void		  removeUserFromList(IMUserDaemon* toRemove);
		
		IMUserDaemon* addUser(ICQUser* pUser, GList** userList = NULL);
	
	private:
		guchar userMask;
		GList	*redirectedQueue;
	
		gchar*	findEventReferrer(CUserEvent *event);
};




// returns the already created owners list
GList *IO_getOwnerList();


// returns a list of all plugins
GList *IO_getPluginsList();


// installs the licq pipe and registers the plugin
void IO_installLicqPipe();

// the pipe callback
gboolean IO_pipeCallback(GIOChannel* source, GIOCondition condition);

#endif /* _IMOWNERDAEMON_H */
