/***************************************************************************
 *            mainwindow.h
 *
 *  Tue Jan 25 22:12:28 2005
 *  Copyright  2005  User
 *  Email
 ****************************************************************************/

#ifndef _MAINWINDOW_H
#define _MAINWINDOW_H

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include "icons.h"
#include "imuserdaemon.h"
#include "settings.h"
#include "contactlist.h"
#include "trayicon.h"
#include "utilities.h"
#include "userutilities.h"
#include "gettext.h"
#include "usertooltips.h"


// things that can be dropped
enum cldropElements
{
	DROP_STRING,
	DROP_FILES,
	DROP_URI,
	DROP_GCONTACT,
	DROP_LCONTACT
};



// different types of menus
enum userMenuGroup
{
	MENU_USER,
	MENU_STATUS,
	MENU_SYSTEM,
	MENU_DOUBLECLICK,
	MENU_NOTIFYBOX,
	MENU_ACCEL
};



// id's for menu elements
enum userMenuID
{
	MENU_OPENEVENT = 0xFFFF,
	MENU_REMOVEUSER,
	MENU_SHOWGROUPS,
	MENU_SHOWEMPTYGROUPS,
	MENU_SHOWOFFLINEUSERS,
	MENU_RELOADLIST,
	MENU_OPENALL,
	MENU_UPDATEALL,
	MENU_CHECKINVISIBLEALL,
	MENU_QUIT,
	MENU_TIPS,
	MENU_ABOUT,
	MENU_NONE	= 0xC0C01A
};


struct statusButtonType
{
	GtkWidget	*button,
				*statusImage,
				*mainMenuEntry;
	GtkMenu		*statusMenu;
	gpointer	owner;
	glong		status;
	guint		connectTimeout;
	gboolean	connectImg;
};



typedef void(*mwMenuClickFunction)(GtkWidget* item, gint menu, gint selection, gpointer data);
typedef void(*mwDropFunction)(contactListEntry* entry, guint type, GtkSelectionData* dragData, gpointer data);

class mainWindow
{
	public:
		GtkWidget	*menu_showEmptyGroups,
					*menu_showGroups,
					*menu_showOfflineUsers,
					*menu_openUserEvent,
					*menu_openAllEvents,
					*menu_userMenuLabel,
					*menu_users,
					*menu_system,
					*menu_users1,
					*menu_system1,
					*menu_users2,
					*menu_system2,
					*menu_syssep,
					*menu_status,
					*menu_reloadContacts,
					*menu_statusInvisible,
					*menu_sendMessage,
					*menu_sendURL,
					*menu_sendFiles,
					*menu_sendChat,
					*menu_sendContacts,
					*menu_sendAuthReq,
					*menu_sendAuth,
					*menu_sendEncryption,
					*statusButton,
					*window;
					
		GList		*statusButtons;
	
	
		contactList	*contacts;	
	
		mainWindow(mwMenuClickFunction mcf, mwDropFunction dcf, GCallback quitfunc, gpointer cbdata);
		~mainWindow();	
	
		void setTitle(gchar* _title);
		void setPendingEventNotify(gboolean state);
	
		void updateStatus(guint status, gboolean isInvisible);
		void updateSystemInfo(gint noe, gchar* evname, gchar* appendix);
		GtkMenu *addStatusButton(IMOwnerDaemon *owner);
		void removeStatusButton(IMOwnerDaemon *owner);
		void updateStatusButton(IMOwnerDaemon *owner, glong status, gboolean invisible);
	
		void startTrayAnimation();
		void stopTrayAnimation();
		gboolean isTrayAnimation();
		void setTrayBlinkEvent(basicEventInfo *ev);
		
		void setLockUserMenus(gboolean lock);
		
		void setWindowTitle(gboolean newEvents);
		
		void destroyWindow();
		
		void showAboutWindow();
	
	private:
	
		GtkWidget	*notifyText,
					*notifyIcon,
					*mainMenu,
					*statusBox,
					*statusButtonIcon,
					*statusButtonText,
					*tooltipWindow;
	
		GtkWidget	*mainMenuContainer,
					*statusBoxContainer,
					*notifyBoxContainer;
	
		GtkMenu		*statusMenu,
					*usersMenu,
					*systemMenu,
					*userMenu,
					*groupMenu,
					*systemUsersMenu;
	
		GdkPixbuf	*lastStatusPixbuf,
					*eventPixbuf;
						
		
		mwMenuClickFunction	menuCallBack;
		gpointer			menuCallBackData;
		
		mwDropFunction		droppingCallback;
	
		trayIcon			*tray;
		gchar				trayIter;
		guint				trayConnectSource,
							trayBlinkSource,
							dragOpenGroupSource,
							lastStatus;
							
		contactListGroup	*dragOpenGroup;
		
		gboolean			trayShowEventIcon,
							lastInvisible,
							showTooltips,
							showMenu,
							showNotify,
							showStatus,
							showTitle,
							isMoving,
							menusLocked,
							titleShowStar,
							titleShowOwners,
							useDoubleClick,
							lastEventState;
							
		gint				moveXDelta,
							moveYDelta;
		
		gchar				*title;
		GString				*statusName,
							*notifyName;

		userTooltips		*tooltips;
		
		GtkAccelGroup	*accelerators;
	
		gboolean	isObscured;
	
		void 		trayClicked(GdkEventType type, guchar button);
		void 		createMenus();
		
		void		createMainMenu();
		void		createNotifyBox();
		void		createStatusBox();
		void		createTrayIcon();
	
	
		// callbacks
		static gboolean cb_contactListClicked(GtkTreeView* cList, GdkEventButton* event, mainWindow* mw);
		static gboolean cb_contactListReleased(GtkTreeView* cList, GdkEventButton* event, mainWindow* self);
		static void	 	cb_contactListActivated(GtkTreeView *tv, GtkTreePath *path, GtkTreeViewColumn *column, mainWindow* mw);

		static void 	cb_contactListElementDropped(mainWindow* self, 
										   			 GdkDragContext   *drag_context,
                                            		 gint              x,
                                            		 gint              y,
                                            		 GtkSelectionData *data,
                                            		 guint             info,
                                            		 guint             time);
		static gboolean cb_contactListDragMotion    (GtkWidget		*widget,
                                            		 GdkDragContext *drag_context,
                                            		 gint            x,
                                            		 gint            y,
                                            		 guint           time,
													 mainWindow		*self);
		static void		cb_contactListDragLeave		(mainWindow* self,
                                            		 GdkDragContext *drag_context,
                                            		 guint           time);
		static gboolean cb_contactListDragOpenGroup (mainWindow* self);
		static void cb_contactListDragDataGet(mainWindow		*self,
                                              GdkDragContext   *drag_context,
                                              GtkSelectionData *data,
                                              guint             info,
                                              guint             time);   
													 
													 
		static void	 	cb_menuClicked(GtkWidget* w, GtkWidget* toshow);
		static void 	cb_menuItemClicked(GtkWidget* item, mainWindow* mw);
		static gboolean	cb_destroyWindow(mainWindow* mw);
		static gboolean cb_visibilityChange(mainWindow* mw, GdkEventVisibility *event);
		static gboolean cb_trayClicked(mainWindow* mw, GdkEventButton* event);
		static gboolean cb_trayAnimation(mainWindow* mw);
		static gboolean cb_trayEventBlinking(mainWindow* mw);
		static gboolean cb_statusButtonBlinking(statusButtonType* but);
		static gboolean cb_statusButtonRightClicked(GtkWidget* but, GdkEventButton* event, mainWindow* self);

		static gboolean cb_windowStateChanged(mainWindow* self, GdkEventWindowState* event);

		static gboolean cb_frameEvent(mainWindow* self, GdkEvent* event);
		
		static gboolean cb_notifyBoxClicked(GtkWidget* box, GdkEventButton* event, mainWindow* mw);

		static void		cb_mainWindowRealized(GtkWidget* w, mainWindow* self);

		static void cb_settingsChangedCallback(gchar* sname, GList *values, mainWindow* self);

		// tooltip callback
		static IMUserDaemon* tooltipFunction(GtkWidget* w, GdkEventMotion* event, mainWindow* self);
		
		// accelerator callbacks (FIXME: every key another accel?)
		static void cb_showOfflineAccelPressed(mainWindow* self);
		static void cb_showGroupsAccelPressed(mainWindow* self);
		static void cb_exitAccelPressed(mainWindow* self);
		static void cb_messageAccelPressed(mainWindow* self);
		static void cb_URLAccelPressed(mainWindow* self);
		static void cb_chatAccelPressed(mainWindow* self);
		static void cb_ftAccelPressed(mainWindow* self);
		static void cb_aResponseAccelPressed(mainWindow* self);
		static void cb_deleteUserFromListAccelPressed(mainWindow* self);
};

#endif /* _MAINWINDOW_H */
