/***************************************************************************
 *            basicwindow.cpp
 *
 *  Fri Jan 21 23:04:35 2005
 *  Copyright  2005  User
 *  Email
 ****************************************************************************/

#include "basicwindow.h"


GList* wc_containerList = NULL;
GList* bw_windowList = NULL;


basicWindow::basicWindow()
{
	window = NULL;
	content = NULL;
	title  = NULL;
	hasFocus = FALSE;
	hasSettingsEntries = FALSE;
	manager = NULL;
	
	sizeX = -1;
	sizeY = -1;
	
	bw_windowList = g_list_append(bw_windowList, this);
	
	wType = UE_UNDEFINED;
	extraSpace = 0;
	
	destroyCallback = NULL;
	destroyCallbackData = NULL;
}


basicWindow::~basicWindow()
{
	if (title)
		g_free(title);
		
	bw_windowList = g_list_remove(bw_windowList, this);
	
	if (hasSettingsEntries)
		settings_getSettings()->removeListenerByData(this);
	
	if (destroyCallback)
		destroyCallback(wType, WC_DESTROYED, 0, this, destroyCallbackData);
}



gboolean basicWindow::createWindow()
{
	GtkWidget		*vbox;
	windowContainer *wc;
	GList			*search;
	gulong			containerMode;
	
	
	if (window)
	{
		focusWindow();
		return TRUE;
	}
	
	
	// create the main window content
	if (!(content = createWindowContent()))
	{
		fprintf(stderr, "basicWindow::createWindow(): FATAL: couldn't create window!\n");
		return FALSE;
	}
	

	settings_getSettings()->getProperties("appearance",
										  "containerMode", &containerMode,
										  NULL);
	
	
	// we are in container mode -> add widget into that container
		if ((containerMode == CM_TYPE) ||
			(containerMode == CM_MESSAGEWINDOW && wType == UE_MESSAGE))
	{
		tabLabel = createContainerTabWidget();
		
		search = wc_containerList;
		
		wc = NULL;
		
		while(search)
		{
			if ( ((windowContainer*)search->data)->cType == wType)
			{
				wc = (windowContainer*)search->data;
				break;
			}
			search = search->next;
		}
		
		if (!wc)
		{
			wc = new windowContainer();
			wc->cType = wType;
		}
		
		wc->addWindow(this);
		
		window = wc->window;
		accelerators = wc->accelerators;
		
		hasFocus = TRUE;
	} else
	// we are not in container mode -> create own window
	{
		// put everything into a vbox
		// vbox = gtk_vbox_new(FALSE,0);
		// gtk_box_pack_start(GTK_BOX(vbox), content, TRUE, TRUE, 0);
	
	
		// create the main window
		window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
		gtk_window_set_title(GTK_WINDOW(window), title);
		// bla = (gpointer)&basicWindow::destroyWindow;
		g_signal_connect_swapped(window, "delete-event", G_CALLBACK(this->cb_destroyWindow), this);
		g_signal_connect_swapped(window, "focus-in-event", G_CALLBACK(this->cb_windowFocusIn), this);
		g_signal_connect_swapped(window, "focus-out-event", G_CALLBACK(this->cb_windowFocusOut), this);

		gtk_container_add(GTK_CONTAINER(window), content);
	
		if (sizeX >= 0)
			gtk_window_set_default_size(GTK_WINDOW(window), sizeX, sizeY);
		
		// create the default ESC accelerator
		accelerators = gtk_accel_group_new();
		gtk_accel_group_connect(accelerators, GDK_Escape, (GdkModifierType)0, GTK_ACCEL_LOCKED, g_cclosure_new_swap(G_CALLBACK(this->cb_escPressed), this, NULL));
		
		gtk_window_add_accel_group(GTK_WINDOW(window), accelerators);		
		
		
		gtk_widget_show_all(window);
		
		windowActivated();
	}
	
	return TRUE;
}



gboolean basicWindow::destroyWindow()
{
	windowContainer *wc;
		
	if (window)
	{
		if (content)
			destroyWindowContent();

		if (wc = (windowContainer*)getWindowContainer())
			wc->removeWindow(this, content);
		else
			gtk_widget_destroy(window);
		window = NULL;
		content = NULL;
	}

	// FIXME: return a real value here
	return TRUE;
}



void basicWindow::windowActivated()
{
	hasFocus = TRUE;
}



void basicWindow::windowDeactivated()
{
	hasFocus = FALSE;
}



void basicWindow::setWindowTitle(gchar* _title)
{
	windowContainer* wc;
	
	title = g_strdup(_title);
		
	if (wc = (windowContainer*)getWindowContainer())
		wc->updateWindowTitle();
	else if (window)
		gtk_window_set_title(GTK_WINDOW(window), title);
}



void basicWindow::setWindowSize(gint _sizeX, gint _sizeY)
{
	sizeX = _sizeX;
	sizeY = _sizeY;
}



GtkWidget* basicWindow::createWindowContent()
{
	return gtk_label_new("You shouldn't be seeing this ...");
}


void basicWindow::destroyWindowContent()
{
	// ...
}



GtkWidget* basicWindow::createContainerTabWidget()
{
	return gtk_label_new("window");
}


void basicWindow::closeWindow()
{
	// ...
/*
	destroyWindowContent();
	content = NULL; */ 
}



void basicWindow::focusWindow()
{
	windowContainer *wc;
	
	
	if (wc = (windowContainer*)getWindowContainer())
		wc->focusWindow(this);
	else
		gtk_window_present(GTK_WINDOW(window));
}



gboolean basicWindow::eventCallback(gint command, gint lastResult, gpointer info)
{
	// ...
	return TRUE;
}



void basicWindow::settingsChangedCallback(gchar* sname, GList *values)
{
	
}



void basicWindow::addSettingsListener(gchar* sname)
{
	settings_getSettings()->installListener(sname, (settingChangedFunc)this->cb_settingsChangedCallback, this, FALSE);
	hasSettingsEntries = TRUE;
}



void basicWindow::allocateExtraSpace(gint toAllocate)
{
	windowContainer* wc;
	gint			 width, height;
	
	if (wc = (windowContainer*)getWindowContainer())
		wc->allocateExtraSpace(toAllocate);
	else
	{
		gtk_window_get_size(GTK_WINDOW(window), &width, &height);
		gtk_window_resize(GTK_WINDOW(window), width + toAllocate, height);
		extraSpace = toAllocate;
	}
}
		


void basicWindow::deallocateExtraSpace()
{
	windowContainer* wc;
	gint			 width, height;
	
	if (wc = (windowContainer*)getWindowContainer())
		wc->deallocateExtraSpace();
	else
	{
		gtk_window_get_size(GTK_WINDOW(window), &width, &height);
		if (width > extraSpace)
			gtk_window_resize(GTK_WINDOW(window), sizeX - extraSpace, sizeY);
	}	
}




gpointer basicWindow::getWindowContainer()
{
	GList	*search;
	windowContainer* wc;
	
		
	search = wc_containerList;
	while(search)
	{
		wc = (windowContainer*)search->data;
		if (wc->containsWindow(this))
		{
			return wc;
		}
		
		search = search->next;
	}
	
	return NULL;
}



void basicWindow::escPressed()
{
	cb_destroyWindow(this);
}




gboolean basicWindow::cb_destroyWindow(basicWindow *bw)
{
	bw->closeWindow();
	bw->destroyWindow();
	delete bw;
	bw = NULL;
	
	return TRUE;
}


gboolean basicWindow::cb_windowFocusOut(basicWindow *bw)
{
	bw->windowDeactivated();
	return FALSE;
}



gboolean basicWindow::cb_windowFocusIn(basicWindow *bw)
{
	bw->windowActivated();
	return FALSE;
}



gboolean basicWindow::cb_eventCallback(userEventType type, gint command, gint lastResult, gpointer info, basicWindow* self)
{
	return self->eventCallback(command, lastResult, info);
}


void basicWindow::cb_settingsChangedCallback(gchar* sname, GList *values, basicWindow* self)
{
	self->settingsChangedCallback(sname, values);
}



void basicWindow::setDestroyCallback(IMEventCallback _dc, gpointer _dcd)
{
	destroyCallback = _dc;
	destroyCallbackData = _dcd;
}



void basicWindow::cb_escPressed(basicWindow* self)
{
	self->escPressed();
}

// =================== windowContainer =========================================




windowContainer::windowContainer()
{
	children = NULL;
	
	noteBook = gtk_notebook_new();
	gtk_notebook_set_scrollable(GTK_NOTEBOOK(noteBook), TRUE);
	gtk_widget_add_events(noteBook, GDK_BUTTON1_MOTION_MASK);
	g_signal_connect(noteBook, "switch-page", G_CALLBACK(this->cb_switchWindow), this);
	g_signal_connect_swapped(noteBook, "button_release_event", G_CALLBACK(this->cb_containerFocusIn), this);
	
	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	g_signal_connect_swapped(window, "delete-event", G_CALLBACK(this->cb_destroyContainer), this);
	g_signal_connect_swapped(window, "focus-in-event", G_CALLBACK(this->cb_containerFocusIn), this);
	g_signal_connect_swapped(window, "focus-out-event", G_CALLBACK(this->cb_containerFocusOut), this);

	
	gtk_container_add(GTK_CONTAINER(window), noteBook);
		
	wc_containerList = g_list_append(wc_containerList, this);
	
	// create the default ESC accelerator
	accelerators = gtk_accel_group_new();
	gtk_accel_group_connect(accelerators, GDK_Escape, (GdkModifierType)0, GTK_ACCEL_LOCKED, g_cclosure_new_swap(G_CALLBACK(this->cb_escPressed), this, NULL));
	gtk_accel_group_connect(accelerators, GDK_Left, GDK_MOD1_MASK, GTK_ACCEL_LOCKED, g_cclosure_new_swap(G_CALLBACK(this->cb_accelLeftPressed), this, NULL));
	gtk_accel_group_connect(accelerators, GDK_Right, GDK_MOD1_MASK, GTK_ACCEL_LOCKED, g_cclosure_new_swap(G_CALLBACK(this->cb_accelRightPressed), this, NULL));
		
	gtk_window_add_accel_group(GTK_WINDOW(window), accelerators);	
	
	extraSpace = 0;
	extraSpaceCount = 0;
}


windowContainer::~windowContainer()
{
	GList	*search;
	basicWindow* wnd;
	

	search = children;
	
	while(search)
	{
		wnd = (basicWindow*)search->data;
		wnd->closeWindow();
		wnd->destroyWindowContent();
		delete wnd;
		search = search->next;
	}
	
	g_list_free(children);
	
	wc_containerList = g_list_remove(wc_containerList, this);
	
	gtk_widget_destroy(window);
}



void windowContainer::addWindow(basicWindow* toAdd)
{
	GtkWidget	*hbox,
				*closeButton,
				*img;
	
	// create the close button next to the tab label
	img = gtk_image_new_from_stock(GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU);
	closeButton = gtk_button_new();
	gtk_container_add(GTK_CONTAINER(closeButton), img);
	gtk_container_set_border_width(GTK_CONTAINER(closeButton), 0);
	gtk_button_set_relief(GTK_BUTTON(closeButton), GTK_RELIEF_NONE);
	
	gtk_widget_set_size_request(closeButton, -1, 16);
	g_object_set_data(G_OBJECT(closeButton), "icqTabID", toAdd);
	g_signal_connect(closeButton, "clicked", G_CALLBACK(this->cb_closeWindowButtonClicked), this);
	
	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(hbox), toAdd->tabLabel, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), closeButton, FALSE, FALSE, 0);
	gtk_widget_show_all(hbox);

	
	children = g_list_append(children, toAdd);
	
	gtk_notebook_append_page(GTK_NOTEBOOK(noteBook), toAdd->content, hbox);
	gtk_widget_show_all(toAdd->content);	
	
	gtk_notebook_set_current_page(GTK_NOTEBOOK(noteBook), -1);
	
	// some initial settings
	if (g_list_length(children)==1)
	{
		if (toAdd->sizeX >= 0)
			gtk_window_resize(GTK_WINDOW(window), toAdd->sizeX, toAdd->sizeY);
		
		gtk_widget_show_all(window);
	}
		
	gtk_window_present(GTK_WINDOW(window));
}



void windowContainer::removeWindow(basicWindow* toRemove, GtkWidget* rcontent)
{
	children = g_list_remove(children, toRemove);
	
	gtk_notebook_remove_page(GTK_NOTEBOOK(noteBook),
							 gtk_notebook_page_num(GTK_NOTEBOOK(noteBook), rcontent));

	if (!g_list_length(children))
		delete this;
}



void windowContainer::focusWindow(basicWindow* toFocus)
{
	gtk_window_present(GTK_WINDOW(window));
	
	gtk_notebook_set_current_page(GTK_NOTEBOOK(noteBook),
								  gtk_notebook_page_num(GTK_NOTEBOOK(noteBook), toFocus->content));

}



gboolean windowContainer::containsWindow(basicWindow* toLook)
{
	GList* res;
	
	res = g_list_find(children, toLook);
	
	return (res!=NULL);
}



void windowContainer::allocateExtraSpace(gint toAllocate)
{
	gint width, height;
	
	extraSpaceCount++;
	
	if (extraSpaceCount == 1)
	{
		gtk_window_get_size(GTK_WINDOW(window), &width, &height);
		gtk_window_resize(GTK_WINDOW(window), width + toAllocate, height);
		
		extraSpace = toAllocate;
	}
}



void windowContainer::deallocateExtraSpace()
{
	gint width, height;
	
	if (!extraSpaceCount)
		return;
	
	extraSpaceCount--;
	if (!extraSpaceCount)
	{
		gtk_window_get_size(GTK_WINDOW(window), &width, &height);
		if (width>extraSpace)
			gtk_window_resize(GTK_WINDOW(window), width - extraSpace, height);
	}
}



void windowContainer::cb_accelLeftPressed(windowContainer* self)
{
	if (!self->noteBook)
		return;
	
	if (gtk_notebook_get_current_page(GTK_NOTEBOOK(self->noteBook)) == 0)
		gtk_notebook_set_current_page(GTK_NOTEBOOK(self->noteBook), 
									  gtk_notebook_get_n_pages(GTK_NOTEBOOK(self->noteBook))-1);
	else
		gtk_notebook_prev_page(GTK_NOTEBOOK(self->noteBook));
}



void windowContainer::cb_accelRightPressed(windowContainer* self)
{
	if (!self->noteBook)
		return;
	
	if (gtk_notebook_get_current_page(GTK_NOTEBOOK(self->noteBook)) == gtk_notebook_get_n_pages(GTK_NOTEBOOK(self->noteBook))-1)
		gtk_notebook_set_current_page(GTK_NOTEBOOK(self->noteBook), 0);
	else
		gtk_notebook_next_page(GTK_NOTEBOOK(self->noteBook));	
}



void windowContainer::cb_destroyContainer(windowContainer* wc)
{
	delete wc;
}



gboolean windowContainer::cb_containerFocusOut(windowContainer* wc)
{
	gint curTab;
	basicWindow *wnd;
	
	curTab = gtk_notebook_get_current_page(GTK_NOTEBOOK(wc->noteBook));
	
	wnd = (basicWindow*)g_list_nth_data(wc->children, curTab);
	
	wnd->windowDeactivated();
	
	return FALSE;
}




gboolean windowContainer::cb_containerFocusIn(windowContainer* wc)
{
	gint curTab;
	basicWindow *wnd;
	
	curTab = gtk_notebook_get_current_page(GTK_NOTEBOOK(wc->noteBook));
	
	wnd = (basicWindow*)g_list_nth_data(wc->children, curTab);
	
	if (wnd)
		wnd->windowActivated();
	
	return FALSE;	
}



void windowContainer::cb_switchWindow(GtkWidget *w, GtkNotebookPage *page, guint newTab, windowContainer* wc)
{
	gint curTab;
	basicWindow *wnd;
	
	curTab = gtk_notebook_get_current_page(GTK_NOTEBOOK(wc->noteBook));
	
	wnd = (basicWindow*)g_list_nth_data(wc->children, curTab);
	
	if (wnd)
		wnd->windowDeactivated();
	
	wnd = (basicWindow*)g_list_nth_data(wc->children, newTab);
	
	if (wnd)
	{
		wnd->windowActivated();
		gtk_window_set_title(GTK_WINDOW(wc->window), wnd->title);	
	} else
		gtk_window_set_title(GTK_WINDOW(wc->window), "");
}



void windowContainer::updateWindowTitle()
{
	gint curTab;
	basicWindow *wnd;
	
	curTab = gtk_notebook_get_current_page(GTK_NOTEBOOK(noteBook));
	
	wnd = (basicWindow*)g_list_nth_data(children, curTab);
		
	if (wnd)
		gtk_window_set_title(GTK_WINDOW(window), wnd->title);
	else
		gtk_window_set_title(GTK_WINDOW(window), "");	
}



void windowContainer::cb_closeWindowButtonClicked(GtkWidget *button, windowContainer* wc)
{
	basicWindow* wnd;
		
	wnd = (basicWindow*)g_object_get_data(G_OBJECT(button), "icqTabID");
		
	// destroy the window -> it will remove itself from the container
	wnd->closeWindow();
	wnd->destroyWindow();
	delete wnd;

}



void windowContainer::cb_closeActiveTab(windowContainer* self)
{
	gint curTab;
	basicWindow *wnd;
	
	curTab = gtk_notebook_get_current_page(GTK_NOTEBOOK(self->noteBook));
	
	wnd = (basicWindow*)g_list_nth_data(self->children, curTab);

	if (!wnd)
		return;

	// destroy the window -> it will remove itself from the container
	wnd->closeWindow();
	wnd->destroyWindow();
	/*
	self->removeWindow(wnd, wnd->content);	
	wnd->closeWindow();
	*/
	delete wnd;
}


void windowContainer::cb_escPressed(windowContainer* self)
{
	gint curTab;
	basicWindow *wnd;
	
	curTab = gtk_notebook_get_current_page(GTK_NOTEBOOK(self->noteBook));
	
	wnd = (basicWindow*)g_list_nth_data(self->children, curTab);

	if (!wnd)
		return;

	wnd->escPressed();
}







GList* wc_getContainers()
{
	return wc_containerList;
}



GList* basicWindow_getAllWindows()
{
	return bw_windowList;
}
