/***************************************************************************
 *            ftwindow.cpp
 *
 *  Thu Feb 10 01:08:09 2005
 *  Copyright  2005  User
 *  Email
 ****************************************************************************/

#include "ftwindow.h"

fileTransferWindow::fileTransferWindow(gboolean _isServer)
{
	isServer = _isServer;
	request = NULL;
	
	filesStore = NULL;
	
	if (isServer)
		wType = UE_FILETRANSFERREQUEST;
	else
		wType = UE_FILETRANSFER;
}




fileTransferWindow::~fileTransferWindow()
{
	if (request)
	{
		request->destroyWindow();
		delete request;
		request = NULL;
	}
	
	if (filesStore)
	{
		gtk_list_store_clear(filesStore);
		g_object_unref(filesStore);
	}
	
	delete manager;
}


void fileTransferWindow::focusWindow()
{
	if (request)
		request->focusWindow();
	else
		basicWindow::focusWindow();
}



void fileTransferWindow::createRequestDialog(fileEventInfo* event)
{	
	gchar	*titleText;
	
	if (isServer)
	{
		request = new requestDialog(TRUE, 
									createFileList(TRUE), 
									tr("Send files to %s" ),
									tr("Please choose the files by pressing "
									"the \"+\" button or by dragging them into the window"),
									NULL, 
									"save.png",
									(requestFunction)this->cb_requestCallback, 
									this);
		
		request->preTitle = g_strdup(tr("Send Files to %s"));

		request->manager = manager;
		request->createWindow();
	} else
	{
		request = new requestDialog(FALSE,
									createRemoteRequestWidget(event),
									tr("Receive files from %s"),
									tr("The user would like to send you one or more files. "
									"By clicking \"Accept\" they will be saved in the directory "
									"specified by the \"Destination\" field."),
									event->text,
									"save.png",
									(requestFunction)this->cb_requestCallback,
									this);

		titleText = g_strdup_printf(tr("Receive Files from %s"), manager->info->alias);
		request->setWindowTitle(titleText);
		g_free(titleText);
		
		request->manager = manager;

		request->createWindow();
	}		
}


		
GtkWidget* fileTransferWindow::createWindowContent()
{
	GtkWidget	*vbox,
				*statusFrame,
				*label,
				*info1Table,
				*info2Table,
				*_info2Frame,
				*info1Frame,
				*info2Frame,
				*vbox2,
				*fileList;
	gchar		*titleText,
				*fstr;
	
	
			
	// create the file selection and the info field
	fileList = createFileList(FALSE);
			
	
	// create single file infos
	info1Table = gtk_table_new(4, 4, FALSE);	
	gtk_table_set_col_spacings(GTK_TABLE(info1Table), 3);
	gtk_table_set_row_spacings(GTK_TABLE(info1Table), 3);
	gtk_container_set_border_width(GTK_CONTAINER(info1Table), 3);	
	
	label = gtk_label_new(tr("file name:"));
	gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.5f);
	info1Name = gtk_entry_new();
	gtk_editable_set_editable(GTK_EDITABLE(info1Name), FALSE);
	gtk_table_attach_defaults(GTK_TABLE(info1Table), label, 0, 1, 0, 1);
	gtk_table_attach_defaults(GTK_TABLE(info1Table), info1Name, 1, 4, 0, 1);

	u_createTableRow(info1Table, tr("Size:"), &info1Sent, tr("Number:"), &info1No, 1);
	// createTableRow(info1Table, "Vergangen:", &info1Elapsed, "Verbleibend:", &info1Remaining, 2);

	label = gtk_label_new(tr("Progress:"));
	gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.5f);
	info1Progress = gtk_progress_bar_new();
	gtk_table_attach_defaults(GTK_TABLE(info1Table), label, 0, 1, 3, 4);
	gtk_table_attach_defaults(GTK_TABLE(info1Table), info1Progress, 1, 4, 3, 4);

	label = gtk_label_new("");
	fstr = g_strdup_printf("<b>%s</b>", tr("Current file:"));
	gtk_label_set_markup(GTK_LABEL(label), fstr);
	g_free(fstr);
	info1Frame = gtk_frame_new(0);
	gtk_frame_set_label_widget(GTK_FRAME(info1Frame), label);
	gtk_container_add(GTK_CONTAINER(info1Frame), info1Table);
	


	// create general info
	info2Table = gtk_table_new(3, 4, FALSE);
	gtk_table_set_col_spacings(GTK_TABLE(info2Table), 3);
	gtk_table_set_row_spacings(GTK_TABLE(info2Table), 3);
	gtk_container_set_border_width(GTK_CONTAINER(info2Table), 3);
	
	u_createTableRow(info2Table, tr("Size:"), &info2Sent, tr("Rate:"), &info2CPS, 0);
	u_createTableRow(info2Table, tr("Elapsed:"), &info2Elapsed, tr("Remaining:"), &info2Remaining, 1);

	label = gtk_label_new(tr("Progress:"));
	gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.5f);
	info2Progress = gtk_progress_bar_new();
	gtk_table_attach_defaults(GTK_TABLE(info2Table), label, 0, 1, 2, 3);
	gtk_table_attach_defaults(GTK_TABLE(info2Table), info2Progress, 1, 4, 2, 3);

	label = gtk_label_new("");
	fstr = g_strdup_printf("<b>%s</b>", tr("Overall progress"));
	gtk_label_set_markup(GTK_LABEL(label), fstr);
	g_free(fstr);
	_info2Frame = gtk_frame_new(0);
	gtk_frame_set_label_widget(GTK_FRAME(_info2Frame), label);
	gtk_container_add(GTK_CONTAINER(_info2Frame), info2Table);

	info2Frame = gtk_alignment_new(0.0f, 0.0f, 1.0f, 1.0f);
	gtk_alignment_set_padding(GTK_ALIGNMENT(info2Frame), 10, 10, 0, 0);
	gtk_container_add(GTK_CONTAINER(info2Frame), _info2Frame);


	// create a vbox for both info fields
	vbox2 = gtk_vbox_new(FALSE,2);
	gtk_box_pack_start(GTK_BOX(vbox2), info1Frame, FALSE, TRUE, 2);
	gtk_box_pack_start(GTK_BOX(vbox2), info2Frame, FALSE, TRUE, 2);
	


	// create the status message
	statusInfo = gtk_label_new(tr("Waiting ..."));
	gtk_misc_set_alignment(GTK_MISC(statusInfo), 0.0f, 0.5f);
	statusFrame = gtk_frame_new(0);
	gtk_frame_set_shadow_type(GTK_FRAME(statusFrame), GTK_SHADOW_IN);
	gtk_container_add(GTK_CONTAINER(statusFrame), statusInfo);
		

	// create the main vbox
	vbox = gtk_vbox_new(FALSE, 2);
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 6);
	
	gtk_box_pack_start(GTK_BOX(vbox), fileList, TRUE, TRUE, 0);
//	gtk_box_pack_start(GTK_BOX(vbox), alignment1, FALSE, TRUE, 0);
	
	gtk_box_pack_start(GTK_BOX(vbox), vbox2, FALSE, TRUE, 2);
	gtk_box_pack_start(GTK_BOX(vbox), statusFrame, FALSE, TRUE, 2);

	gtk_box_pack_start(GTK_BOX(vbox), gtk_hseparator_new(), FALSE, TRUE, 2);
	gtk_box_pack_start(GTK_BOX(vbox), createButtonbar(), FALSE, TRUE, 2);

	if (manager)
	{
		titleText = g_strdup_printf(tr("File transfer with %s"), manager->info->alias);
		setWindowTitle(titleText);
		g_free(titleText);
	}


	return vbox;
}



void fileTransferWindow::destroyWindowContent()
{

}



GtkWidget* fileTransferWindow::createRemoteRequestWidget(fileEventInfo* event)
{
	GtkWidget	*label1,
				*label2,
				*label3,
				*entry1,
				*entry2,
				*hbox,
				*btn,
				*table;
				
	gchar		*txt,
				*lastDirectory;
		
			
	label1 = gtk_label_new(tr("Name:"));
	gtk_misc_set_alignment(GTK_MISC(label1), 0.0f, 0.5f);
	label2 = gtk_label_new(tr("Size:"));
	gtk_misc_set_alignment(GTK_MISC(label2), 0.0f, 0.5f);
	gtk_misc_set_padding(GTK_MISC(label2), 5, 0);
	label3 = gtk_label_new(tr("Destination:"));
	gtk_misc_set_alignment(GTK_MISC(label3), 0.0f, 0.5f);
	
	entry1 = gtk_entry_new();
	gtk_widget_set_size_request(entry1, 150, -1);
	entry2 = gtk_entry_new();
	gtk_widget_set_size_request(entry2, 70, -1);	
	

	gtk_entry_set_text(GTK_ENTRY(entry1), event->fileName);
	
	txt = u_convertSize2Readable(event->fileSize);
	gtk_entry_set_text(GTK_ENTRY(entry2), txt);
	g_free(txt);

	

	dirEntry = gtk_entry_new();
	settings_getSettings()->getProperties("filetransfer",
										  "lastDirectory", &lastDirectory,
										  NULL);
	gtk_entry_set_text(GTK_ENTRY(dirEntry), lastDirectory);
	
	btn = u_createStockImageButton("gtk-directory");
	gtk_button_set_relief(GTK_BUTTON(btn), GTK_RELIEF_HALF);
	// g_signal_connect(btn, "clicked", G_CALLBACK(rWdirectoryButtonClicked), this);
	
	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(hbox), dirEntry, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), btn, FALSE, FALSE, 0);
	
	
	
	
	// put everything together
	table = gtk_table_new(2,4,FALSE);
	gtk_container_set_border_width(GTK_CONTAINER(table), 0);
	gtk_table_set_row_spacings(GTK_TABLE(table), 2);
	gtk_table_attach_defaults(GTK_TABLE(table), label1, 0, 1, 0, 1);
	gtk_table_attach_defaults(GTK_TABLE(table), entry1, 1, 2, 0, 1);

	gtk_table_attach_defaults(GTK_TABLE(table), label2, 2, 3, 0, 1);
	gtk_table_attach_defaults(GTK_TABLE(table), entry2, 3, 4, 0, 1);
	
	gtk_table_attach_defaults(GTK_TABLE(table), label3, 0, 1, 1, 2);
	gtk_table_attach_defaults(GTK_TABLE(table), hbox, 1, 4, 1, 2);


	return table;
}



GtkWidget* fileTransferWindow::createButtonbar()
{
	GtkWidget	*hbox;
	
	closeButton = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
	g_signal_connect_swapped(closeButton, "clicked", G_CALLBACK(this->cb_closeButtonClicked), this);
	
	hbox = gtk_hbox_new(FALSE, 2);
	
	gtk_box_pack_end(GTK_BOX(hbox), closeButton, FALSE, FALSE, 0);
	
	return hbox;
}

/*
gboolean hasDragEntered = FALSE;
GtkTreePath *dragPath = NULL;


static gboolean
drag_motion (GtkWidget *widget,
      	  GdkDragContext *context,
             gint x,
             gint y,
             guint time,
			GtkWidget* toHighlight)
{
	GtkTreePath	*path;
	GtkTreeIter iter,
				oldIter;
	GtkTreeModel	*model;
	
	if (!hasDragEntered)
	{
		hasDragEntered = TRUE;
		gtk_drag_highlight(toHighlight);
	}
	
	
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(widget));
	
	// the mouse is on an entry
	gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(widget), x, y, &path, NULL, NULL, NULL);
		
	// did our position change?
	if (dragPath &&
		!(!path && gtk_tree_path_get_indices(dragPath)[0] == (gtk_tree_model_iter_n_children(model, NULL)-1)) && 		
		(!path || gtk_tree_path_compare(path, dragPath) != 0))
	{
		gtk_tree_model_get_iter(model, &iter, dragPath);
		gtk_tree_store_remove(GTK_TREE_STORE(model), &iter);
		gtk_tree_path_free(dragPath);
		dragPath = NULL;
	}
	
	if (path)
		gtk_tree_path_free(path);
	
	// there is no current selection -> create one
	if (!dragPath)
	{
		gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(widget), x, y, &path, NULL, NULL, NULL);
		
		// create a new selection or append it
		if (path)
		{
			gtk_tree_model_get_iter(model, &oldIter, path);
			gtk_tree_store_insert_before(GTK_TREE_STORE(model), &iter, NULL, &oldIter);		
			gtk_tree_path_free(path);
		} else
			gtk_tree_store_append(GTK_TREE_STORE(model), &iter, NULL);
		
		dragPath = gtk_tree_model_get_path(model, &iter);
		gtk_tree_store_set(GTK_TREE_STORE(model), &iter, FTCOL_FILENAME, "Halloballo", -1);
		gtk_tree_view_set_drag_dest_row(GTK_TREE_VIEW(widget), dragPath, GTK_TREE_VIEW_DROP_INTO_OR_AFTER);
		gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(widget), dragPath, NULL, FALSE, 0, 0);
	} 
	
 
	return TRUE;
}


void        drag_leave                  (GtkWidget      *widget,
                                            GdkDragContext *drag_context,
                                            guint           time,
                                            GtkWidget	*toHighlight)   
{
	GtkTreeIter iter;
	GtkTreeModel *model;
	
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(widget));
	
	if (hasDragEntered)
	{
		gtk_drag_unhighlight(toHighlight);
		hasDragEntered = FALSE;
	}
	
	if (dragPath)
	{
		gtk_tree_model_get_iter(model, &iter, dragPath);
		gtk_tree_store_remove(GTK_TREE_STORE(model), &iter);
		gtk_tree_path_free(dragPath);
		dragPath = NULL;
	}
		
}
  */




void fileTransferWindow::addFilesFromURIList(gchar* URI, GtkTreeIter* whereToAdd)
{
	gchar		*currentPosition,
				*nextPosition,
				*filename,
				*hostname,
				*currentStrFixed,
				*workURI;
	gboolean	showNotLocalWarning;
	
	
	// don't do anything if we didn't start the request ...
	if (!request || !request->isServer)
		return;
	
	// process the string and find every file name (the data can consist of many files)
	workURI = g_strdup(URI);
	currentPosition = workURI;
	
	showNotLocalWarning = FALSE;
	
	while( (nextPosition = g_strstr_len(currentPosition, strlen(currentPosition), "\n")) )
	{
		nextPosition[0] = '\0';		

		if (currentPosition[nextPosition-currentPosition-1] == '\r')
			currentPosition[nextPosition-currentPosition-1] = '\0';
		
		filename = g_filename_from_uri(currentPosition, &hostname, NULL);
		
		// stolen code - I love open source :)
		if (!filename && !hostname)
		{
			currentStrFixed = g_strconcat("file:", currentPosition, NULL);
			filename = g_filename_from_uri(currentStrFixed, &hostname, NULL);
			g_free(currentStrFixed);
		}
		
		if (filename)
		{			
			if (hostname)
			{
				showNotLocalWarning = TRUE;
				g_free(hostname);
			} else
				addFile(filename, whereToAdd);

			g_free(filename);			
		}
				
		currentPosition = nextPosition+1;		
	}
	
	
	setFileInfo();
	
	if (showNotLocalWarning)
		u_showAlertMessage(tr("Cannot send not-local files"), tr("You tried to add a file that is not local. Currently Licq doesn't support this feature - Sorry"), GTK_STOCK_DIALOG_ERROR);

	g_free(workURI);	
}






void fileTransferWindow::cb_fileListFileDropped(fileTransferWindow* ftw, 
										   		GdkDragContext   *drag_context,
                                            	gint              x,
                                            	gint              y,
                                            	GtkSelectionData *data,
                                            	guint             info,
                                           		guint             time)
{
	GtkTreeIter	iter;
	GtkTreePath	*path;
	
	gtk_tree_view_get_dest_row_at_pos(GTK_TREE_VIEW(ftw->filesTv), x, y, &path, NULL);
	
	if (path)
		gtk_tree_model_get_iter(GTK_TREE_MODEL(ftw->filesStore), &iter, path);

	ftw->addFilesFromURIList((gchar*)data->data, path? &iter: NULL);	
	
	if (path)
	{
		gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(ftw->filesTv), path, NULL, FALSE, 0, 0);
		gtk_tree_path_free(path);
	}
}




GtkWidget* fileTransferWindow::createFileList(gboolean createButtons)
{
	GtkCellRenderer		*renderer;
	GtkTreeViewColumn	*column;
	GtkWidget			*scroll,
						*addButton,
						*removeButton,
						*buttonImg,
						*hbox,
						*vbox,
						*vbox2,
						*alignment1;
	
	static const GtkTargetEntry targets[] = 
	{
		{"GTK_TREE_MODEL_ROW", 0, 1},
		{"text/uri-list", 0, 1}
	};
	
		
	
	// create the tree view 
	// -----------------------------
	if (!filesStore)
		filesStore = gtk_list_store_new(5, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN, G_TYPE_INT);	
	

	filesTv = gtk_tree_view_new_with_model(GTK_TREE_MODEL(filesStore));
	gtk_tree_view_set_rules_hint(GTK_TREE_VIEW (filesTv), TRUE);
	gtk_tree_selection_set_mode(gtk_tree_view_get_selection (GTK_TREE_VIEW (filesTv)),
								GTK_SELECTION_MULTIPLE);
	
	
	// 1. file name
	renderer = gtk_cell_renderer_text_new();
	g_object_set(renderer, "xalign", 0.0, NULL);
	column = gtk_tree_view_column_new_with_attributes(tr("File name"), renderer, 
													  "markup", FTCOL_FILENAME,
													  NULL );
	gtk_tree_view_column_set_min_width(column, 300);
	gtk_tree_view_append_column(GTK_TREE_VIEW(filesTv), column);

	
	// 2. file size
	renderer = gtk_cell_renderer_text_new();
	g_object_set(renderer, "xalign", 1.0, NULL);
	column = gtk_tree_view_column_new_with_attributes(tr("Size"), renderer, 
													  "markup", FTCOL_SIZESTR,
													  NULL );	
	gtk_tree_view_append_column(GTK_TREE_VIEW(filesTv), column);
	
	
	// create a scrolled window around it
	scroll = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll),
								   GTK_POLICY_AUTOMATIC,
								   GTK_POLICY_AUTOMATIC );
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scroll), GTK_SHADOW_IN);
	gtk_container_add(GTK_CONTAINER(scroll), filesTv);

	
	// set the filesTv as a drag&drop destination
	gtk_tree_view_enable_model_drag_dest(GTK_TREE_VIEW(filesTv), targets, 2, (GdkDragAction)(GDK_ACTION_COPY | GDK_ACTION_MOVE));
	gtk_tree_view_enable_model_drag_source(GTK_TREE_VIEW(filesTv), GDK_BUTTON1_MASK, targets, 2, (GdkDragAction)(GDK_ACTION_COPY | GDK_ACTION_MOVE));
	g_signal_connect_swapped(filesTv, "drag-data-received", G_CALLBACK(cb_fileListFileDropped), this);
/*gtk_drag_dest_set(filesTv, GTK_DEST_DEFAULT_ALL, targets, 2, (GdkDragAction)(GDK_ACTION_COPY | GDK_ACTION_MOVE));
	g_signal_connect(filesTv, "drag-motion", G_CALLBACK(drag_motion), scroll);
	g_signal_connect(filesTv, "drag-leave", G_CALLBACK(drag_leave), scroll);
	*/
	
	
	// create small buttons
	// --------------------------------------------
	if (createButtons)
	{
		// 1. "add file" - button
		buttonImg = gtk_image_new_from_stock(GTK_STOCK_ADD, GTK_ICON_SIZE_BUTTON);
		addButton = gtk_button_new();
		gtk_container_add(GTK_CONTAINER(addButton), buttonImg);
		g_signal_connect_swapped(addButton, "clicked", G_CALLBACK(this->cb_addFileButtonClicked), this);
	
		// 2. "remove file" - button
		buttonImg = gtk_image_new_from_stock(GTK_STOCK_REMOVE, GTK_ICON_SIZE_BUTTON);
		removeButton = gtk_button_new();
		gtk_container_add(GTK_CONTAINER(removeButton), buttonImg);
		g_signal_connect_swapped(removeButton, "clicked", G_CALLBACK(this->cb_removeFileButtonClicked), this);
	
		// 3. "go up" - button
		buttonImg = gtk_image_new_from_stock(GTK_STOCK_GO_UP, GTK_ICON_SIZE_BUTTON);
		fileUpButton = gtk_button_new();
		gtk_container_add(GTK_CONTAINER(fileUpButton), buttonImg);
		g_signal_connect_swapped(fileUpButton, "clicked", G_CALLBACK(this->cb_moveFileUpButtonClicked), this);
		gtk_widget_set_sensitive(fileUpButton, FALSE);
		
		// 4. "go down" - button
		buttonImg = gtk_image_new_from_stock(GTK_STOCK_GO_DOWN, GTK_ICON_SIZE_BUTTON);
		fileDownButton = gtk_button_new();
		gtk_container_add(GTK_CONTAINER(fileDownButton), buttonImg);
		g_signal_connect_swapped(fileDownButton, "clicked", G_CALLBACK(this->cb_moveFileDownButtonClicked), this);
		gtk_widget_set_sensitive(fileDownButton, FALSE);
	
		vbox = gtk_vbox_new(FALSE, 2);
		gtk_box_pack_start(GTK_BOX(vbox), addButton, FALSE, FALSE, 0);
		gtk_box_pack_start(GTK_BOX(vbox), removeButton, FALSE, FALSE, 0);
		gtk_box_pack_start(GTK_BOX(vbox), gtk_hseparator_new(), FALSE, TRUE, 2);
		gtk_box_pack_start(GTK_BOX(vbox), fileUpButton, FALSE, FALSE, 0);
		gtk_box_pack_start(GTK_BOX(vbox), fileDownButton, FALSE, FALSE, 0);
	
		// add a signal for the up and down button (to get grayed out eventually)
		g_signal_connect_swapped(gtk_tree_view_get_selection(GTK_TREE_VIEW(filesTv)), "changed", G_CALLBACK(this->cb_fileListCursorChanged), this);
	
		// add everything in a big hbox
		// (only add small buttons when they are needed)		
		hbox = gtk_hbox_new(FALSE, 2);
		gtk_box_pack_start(GTK_BOX(hbox), scroll, TRUE, TRUE, 0);
		gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, TRUE, 0);	
	} else
		hbox = scroll;
	
	
	// create a vbox to pack the new box + info label
	
	vbox2 = gtk_vbox_new(FALSE, 2);
	
	filesInfo = gtk_label_new("");
	gtk_misc_set_alignment(GTK_MISC(filesInfo), 0.0f, 0.5f);
	
	alignment1 = gtk_alignment_new(0.0f, 0.0f, 1.0f, 1.0f);
	gtk_alignment_set_padding(GTK_ALIGNMENT(alignment1), 0, 10, 0, 0);
	gtk_container_add(GTK_CONTAINER(alignment1), filesInfo);
	
	
	gtk_box_pack_start(GTK_BOX(vbox2), hbox, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox2), alignment1, FALSE, TRUE, 0);
	
	setFileInfo();
	
	return vbox2;

}



void fileTransferWindow::setFileInfo()
{
	GtkTreeIter iter;
	gint		size, 
				oneSize;
	gchar		*toShow, *appendix, *sizeStr;
	
	size = 0;
	
	
	if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(filesStore), &iter))
	{
		do
		{
			gtk_tree_model_get(GTK_TREE_MODEL(filesStore), &iter,
							   FTCOL_SIZE, &oneSize,
							   -1);
			
			size+=oneSize;
		} while(gtk_tree_model_iter_next(GTK_TREE_MODEL(filesStore), &iter));

	}
	
	if (gtk_tree_model_iter_n_children(GTK_TREE_MODEL(filesStore), NULL) == 1)
		appendix = "";
	else
		appendix = tr("s");
	
	sizeStr = u_convertSize2Readable(size);
	
	toShow = g_strdup_printf(tr("%d file%s have a size of %s"), 
							 gtk_tree_model_iter_n_children(GTK_TREE_MODEL(filesStore), NULL),
							 appendix, sizeStr);
		
	
	gtk_label_set_markup(GTK_LABEL(filesInfo), toShow);
	
	
	g_free(sizeStr);
	g_free(toShow);	
}



void fileTransferWindow::showAddFileDialog()
{
	GtkWidget*	fileSelection;
	gint		ret;
	GSList		*files,
				*search;

	
	fileSelection = gtk_file_chooser_dialog_new(tr("Select files to send"), 
												NULL,
												GTK_FILE_CHOOSER_ACTION_OPEN,
				     							GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
				      							GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
												NULL);
	gtk_file_chooser_set_select_multiple(GTK_FILE_CHOOSER(fileSelection), TRUE);
	gtk_file_chooser_set_local_only(GTK_FILE_CHOOSER(fileSelection), TRUE);
	
	ret = gtk_dialog_run (GTK_DIALOG (fileSelection));
 
	if (ret != GTK_RESPONSE_ACCEPT)
	{
		gtk_widget_destroy(fileSelection);
		return;
	}
	
	files = gtk_file_chooser_get_filenames(GTK_FILE_CHOOSER(fileSelection));
	
	for (search = files; search; search = search->next)
	{
		addFile((gchar*)search->data);
		g_free(search->data);
	}
	g_slist_free(files);
	
	// update info label
	setFileInfo();
	
	gtk_widget_destroy(fileSelection);

}



gboolean fileTransferWindow::addFile(gchar* fileName, GtkTreeIter* iterToInsert)
{
	gchar	    *oneName, 
			    *tmp;
	struct stat fInfo;
		
	GtkTreeIter iter;

	
	// 1. find file size
		
	if (stat(fileName, &fInfo)!= 0)
	{
		tmp = g_strdup_printf(tr_NOOP("The file %s\ncouldn't be read. Please check your permissions on that file!\n"), fileName);
		u_showAlertMessage(tr("Error at opening file"), tr(tmp), GTK_STOCK_DIALOG_ERROR);
		g_free(tmp);
		return FALSE;
	}
	
	// check whether this IS a file
	if (!S_ISREG(fInfo.st_mode))
	{
		tmp = g_strdup_printf(tr_NOOP("%s doesn't appear to be a regular file. Please only add normal files."), fileName);
		u_showAlertMessage(tr("No file"), tr(tmp), GTK_STOCK_DIALOG_ERROR);
		g_free(tmp);
		return FALSE;
	}

		
	// 2. check whether the file is already in the list
	
	if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(filesStore), &iter))
	{
		do
		{
			gtk_tree_model_get(GTK_TREE_MODEL(filesStore), &iter, FTCOL_FILENAME, &oneName, -1);
			if (strcmp(fileName, oneName)==0)
			{
				tmp = g_strdup_printf(tr_NOOP("The file %s\nis already in the list!\n"), fileName);
				u_showAlertMessage(tr("file already in list"), tr(tmp), GTK_STOCK_DIALOG_WARNING);
				g_free(tmp);					
				return FALSE;
				break;
			}
		} while(gtk_tree_model_iter_next(GTK_TREE_MODEL(filesStore), &iter));
			
	}
		

	// 3. add to the store
		
	tmp = u_convertSize2Readable(fInfo.st_size);
		
	if (iterToInsert)
		gtk_list_store_insert_before(filesStore, &iter, iterToInsert);
	else
		gtk_list_store_append(filesStore, &iter);
	
	gtk_list_store_set(filesStore, &iter,
						FTCOL_FILENAME, fileName,
						FTCOL_SIZE	  , fInfo.st_size,
						FTCOL_SIZESTR , tmp,
						FTCOL_SELECTED, FALSE,
						-1);
	
	g_free(tmp);


	return TRUE;	
}



void fileTransferWindow::removeSelectedFiles()
{
	GtkTreeIter		 *iter;
	GtkTreeSelection *ts;
	GList			 *sels, 
					 *search, 
					 *iters;
	
	// get our selection
	ts = gtk_tree_view_get_selection(GTK_TREE_VIEW(filesTv));
	
	sels = gtk_tree_selection_get_selected_rows(GTK_TREE_SELECTION(ts), NULL);
	
	search = sels;
	
	iters = NULL;
	
	while(search)
	{
		iter = (GtkTreeIter*)g_malloc(sizeof(GtkTreeIter));
		gtk_tree_model_get_iter(GTK_TREE_MODEL(filesStore), iter, 
								(GtkTreePath*)search->data);
		
		iters = g_list_append(iters, iter);
		search = search->next;
	}
	
	search = iters;
	while (search)
	{
		gtk_list_store_remove(filesStore, (GtkTreeIter*)search->data);
		g_free(search->data);
		search = search->next;
	}

	g_list_foreach(sels, (GFunc)gtk_tree_path_free, NULL);
	g_list_free(sels);
	g_list_free(iters);	
	
	
	setFileInfo();
}




void fileTransferWindow::moveSelectedFiles(gboolean moveUp)
{
	GtkTreeIter		 iter1, iter2;
	GtkTreeSelection *ts;
	GList			 *sels, 
					 *search;
		
	// get our selection
	ts = gtk_tree_view_get_selection(GTK_TREE_VIEW(filesTv));
	
	sels = gtk_tree_selection_get_selected_rows(GTK_TREE_SELECTION(ts), NULL);
	
	if (moveUp)
		search = sels;
	else
		search = g_list_last(sels);
	
	while(search)
	{

		gtk_tree_model_get_iter(GTK_TREE_MODEL(filesStore), &iter1, 
								(GtkTreePath*)search->data);
		
		if (moveUp)
			gtk_tree_path_prev((GtkTreePath*)search->data);
		else
			gtk_tree_path_next((GtkTreePath*)search->data);
		
		gtk_tree_model_get_iter(GTK_TREE_MODEL(filesStore), &iter2,
								(GtkTreePath*)search->data);
		
		gtk_list_store_swap(filesStore, &iter1, &iter2);
		
		if (moveUp)
			search = search->next;
		else
			search = search->prev;
	}
		
	g_list_foreach(sels, (GFunc)gtk_tree_path_free, NULL);
	g_list_free(sels);
}





gboolean fileTransferWindow::eventCallback(gint command, gint lastResult, gpointer info)
{
	fileEventInfo *fileInfo;
	gchar		  *text, 
				  *text2, 
				  *text3;
	GtkTreeIter   iter;
	
	switch(command)
	{
		case UC_EVENT_RECEIVED:
			if (!isServer && !request)
				createRequestDialog((fileEventInfo*)info);
			break;
			
		case EVENT_REQUEST_ACCEPTED:
			request->destroyWindow();
			request = NULL;
			createWindow();
			break;
		
		case EVENT_REQUEST_REFUSED:
			text = g_strdup_printf(tr_NOOP("Your request was refused:\n%s"), (gchar*)info);
			u_showAlertMessage(tr("Refused!"), tr(text), GTK_STOCK_DIALOG_INFO);
			g_free(text);

			request->destroyWindow();
			delete request;
			request = NULL;
			delete this;
			break;
		
		case UC_EVENT_FINISHED:
			if (lastResult != EVENT_RESULT_SUCCESS && lastResult != EVENT_RESULT_ACKED)
			{
				uu_showSendErrorMessage(tr("Could not send files"), tr("Sending of the request to send files failed"), lastResult, manager->parentUser->owner->info->status);
				request->stopWaitForRequestAnswer();
			}
			break;
		
		case FEVENT_RES_STARTFILE:
		case FEVENT_RES_CONFIRM:	
		case FEVENT_RES_STARTBATCH: 
		case FEVENT_RES_DONEFILE: 
		case FEVENT_RES_DONEBATCH:
		case FEVENT_RES_UPDATE:	
			// update info window
		
			fileInfo = (fileEventInfo*)info;
		
			gtk_entry_set_text(GTK_ENTRY(info1Name), fileInfo->fileName);
		
			text = g_strdup_printf("%d/%d", fileInfo->batchFile, fileInfo->batchCount);
			gtk_entry_set_text(GTK_ENTRY(info1No), text);
			g_free(text);

			text2 = u_convertSize2Readable(fileInfo->filePosition);
			text3 = u_convertSize2Readable(fileInfo->fileSize);
			text = g_strdup_printf("%s %s", text2, text3);
			gtk_entry_set_text(GTK_ENTRY(info1Sent), text);
			g_free(text); g_free(text2); g_free(text3);
		
			gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(info1Progress), fileInfo->fileProgress);

			text2 = u_convertSize2Readable(fileInfo->batchPosition);
			text3 = u_convertSize2Readable(fileInfo->batchSize);
			text = g_strdup_printf("%s/%s", text2, text3);
			gtk_entry_set_text(GTK_ENTRY(info2Sent), text);
			g_free(text); g_free(text2); g_free(text3);

			text2 = u_convertSize2Readable((gint)fileInfo->averageSpeed);
			text = g_strdup_printf("%s/s", text2);
			gtk_entry_set_text(GTK_ENTRY(info2CPS), text);
			g_free(text2); g_free(text);
			
			text = g_strdup_printf("%02ld:%02ld:%02ld",
				fileInfo->elapsed / 3600, (fileInfo->elapsed % 3600) / 60, (fileInfo->elapsed % 60) );
			gtk_entry_set_text(GTK_ENTRY(info2Elapsed), text);
			g_free(text);

			text = g_strdup_printf("%02ld:%02ld:%02ld",
				fileInfo->remaining / 3600, (fileInfo->remaining % 3600) / 60, (fileInfo->remaining % 60) );
			gtk_entry_set_text(GTK_ENTRY(info2Remaining), text);
			g_free(text);
			
			gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(info2Progress), fileInfo->batchProgress);

			if (command == FEVENT_RES_DONEBATCH)
			{
				stopFileTransfer();
				gtk_label_set_text(GTK_LABEL(statusInfo), tr("files transferred successfully"));
			} else
			if (command == FEVENT_RES_STARTBATCH)
			{
				gtk_label_set_text(GTK_LABEL(statusInfo), tr("transferring files ..."));
			} 
			
			// add file to the list (receiver only)
			if (command == FEVENT_RES_DONEFILE && !isServer)
			{
				text = u_convertSize2Readable(fileInfo->fileSize);
				gtk_list_store_append(filesStore, &iter);
				gtk_list_store_set(filesStore, &iter, 
									FTCOL_FILENAME, fileInfo->fileName,
									FTCOL_SIZESTR, text,
									-1);	
				g_free(text);
			}

			break;			
	}
	
	if (request)
		return request->eventCallback(command, lastResult, info);
	else
		return TRUE;
}



void fileTransferWindow::stopFileTransfer()
{
	gtk_button_set_label(GTK_BUTTON(closeButton), GTK_STOCK_CLOSE);
	gtk_button_set_use_stock(GTK_BUTTON(closeButton), TRUE);
	
	manager->cancelInitatedEvent();
}


void fileTransferWindow::cb_requestCallback(gint result, gpointer info, fileTransferWindow* ftw)
{
	GList* toSend;
	GtkTreeIter iter;
	gchar* oneName,*dest;
	
	
	if (ftw->isServer)
	{
		switch(result)
		{
			// send the request
			case REQUEST_ACCEPTED:
				// create a GList out of the GtkTreeView
				toSend = NULL;
				if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(ftw->filesStore), &iter))
				{
					do
					{
						gtk_tree_model_get(GTK_TREE_MODEL(ftw->filesStore), &iter, FTCOL_FILENAME, &oneName, -1);
						toSend = g_list_append(toSend, oneName);
					} while(gtk_tree_model_iter_next(GTK_TREE_MODEL(ftw->filesStore), &iter));
				}	
			
				if (!toSend)
				{
					u_showAlertMessage(tr("nothing to send"), tr("Please select the files to send"), GTK_STOCK_DIALOG_WARNING);
					return;
				}

				((IMFileTransferManager*)ftw->manager)->sendRequest((gchar*)info, toSend);			

				ftw->request->waitForRequestAnswer();
				break;
			
		
			// window was destroyed without sending any request
			case REQUEST_CLOSED:
				ftw->request = NULL;
				delete ftw;
				break;
			
			case REQUEST_CANCELLED:
				ftw->request->stopWaitForRequestAnswer();
				ftw->manager->cancelInitatedEvent();
				break;
		}

	} else
	{
		switch(result)
		{
			// window was destroyed without any response
			case REQUEST_CLOSED:
				ftw->request = NULL;
				delete ftw;
				break;
		
			// request was refused
			case REQUEST_REFUSED:
				((IMFileTransferManager*)ftw->manager)->answerRemoteRequest(FALSE, (gchar*)info);
				ftw->request->destroyWindow();
				delete ftw->request;
				ftw->request = NULL;
				delete ftw;
				break;
		
			// request was accepted
			case REQUEST_ACCEPTED:
				dest = (gchar*)gtk_entry_get_text(GTK_ENTRY(ftw->dirEntry));
				if (!g_file_test(dest, G_FILE_TEST_EXISTS))
					u_showAlertMessage(tr("Error"), tr("The specified directory\ndoesn't exist!"), GTK_STOCK_DIALOG_ERROR);
				else
				if (!g_file_test(dest, G_FILE_TEST_IS_DIR))
					u_showAlertMessage(tr("Error"), tr("The specified destination\nis NOT a directory!"), GTK_STOCK_DIALOG_ERROR);
				else
				{				
					ftw->createWindow();
					((IMFileTransferManager*)ftw->manager)->answerRemoteRequest(TRUE, dest);

					ftw->request->destroyWindow();
					delete ftw->request;
					ftw->request = NULL;
				}
				break;
		}		
	}
}





void fileTransferWindow::cb_addFileButtonClicked(fileTransferWindow* ftw)
{
	ftw->showAddFileDialog();
}



void fileTransferWindow::cb_removeFileButtonClicked(fileTransferWindow* ftw)
{
	ftw->removeSelectedFiles();
}



void fileTransferWindow::cb_moveFileUpButtonClicked(fileTransferWindow* ftw)
{
	ftw->moveSelectedFiles(TRUE);
	ftw->cb_fileListCursorChanged(ftw);
}




void fileTransferWindow::cb_moveFileDownButtonClicked(fileTransferWindow* ftw)
{
	ftw->moveSelectedFiles(FALSE);
	ftw->cb_fileListCursorChanged(ftw);	
}


void fileTransferWindow::cb_fileListCursorChanged(fileTransferWindow* ftw)
{
	GtkTreeSelection *ts;
	GList			 *sels;
		
	// get our selection
	ts = gtk_tree_view_get_selection(GTK_TREE_VIEW(ftw->filesTv));
	
	sels = gtk_tree_selection_get_selected_rows(GTK_TREE_SELECTION(ts), NULL);

	if (!sels)
		return;
	

	if (gtk_tree_path_get_indices((GtkTreePath*)sels->data)[0] == 0)
		gtk_widget_set_sensitive(ftw->fileUpButton, FALSE);
	else
		gtk_widget_set_sensitive(ftw->fileUpButton, TRUE);
	
	if (gtk_tree_path_get_indices((GtkTreePath*)g_list_last(sels)->data)[0] ==
		gtk_tree_model_iter_n_children(GTK_TREE_MODEL(ftw->filesStore), NULL)-1)
		gtk_widget_set_sensitive(ftw->fileDownButton, FALSE);
	else
		gtk_widget_set_sensitive(ftw->fileDownButton, TRUE);
	
	g_list_foreach(sels, (GFunc)gtk_tree_path_free, NULL);
	g_list_free(sels);
}





void fileTransferWindow::cb_closeButtonClicked(fileTransferWindow* ftw)
{
	if (((IMFileTransferManager*)ftw->manager)->isTransferring )
		ftw->stopFileTransfer();
	else
	{
		ftw->destroyWindow();
		delete ftw;
	}
}
