/***************************************************************************
 *            trayicon.cpp
 *
 *  Tue Sep  7 01:10:19 2004
 *  Copyright  2004  User
 *  Email
 ****************************************************************************/

#include "trayicon.h"


trayIcon::trayIcon()
{
	trayWindow = None;
}



trayIcon::~trayIcon()
{
	
}



gboolean trayIcon::installTrayIcon(GtkWidget* icon)
{
	GdkDisplay 		*gdkDisplay;
	GdkScreen		*gdkScreen;
	gchar			*tmp;
	
	
	// 1. create a new window (that will be embedded into the tray)
	iconWidget = icon;

	createTrayIconWindow();
	
	// 2. get display and screen information and atoms for neccessary X-functions
	// FIXME: not multihead-compatible ?
	gdkDisplay = gtk_widget_get_display(pluggedIcon);
	xdisplay = GDK_DISPLAY_XDISPLAY(gdkDisplay);
	gdkScreen = gdk_screen_get_default ();
	tmp = g_strdup_printf("_NET_SYSTEM_TRAY_S%d", GDK_SCREEN_XNUMBER(gdkScreen));

	systemTraySelection = XInternAtom(xdisplay, tmp, False);
	xManager 			= XInternAtom(xdisplay, "MANAGER", False);
	managerCommand	    = XInternAtom(xdisplay, "_NET_SYSTEM_TRAY_OPCODE", False);

	g_free(tmp);
	
	// 3. get the system tray window
	getSystemTrayWindow();
	
	if (trayWindow != None)
	{

		// 4. now ask system tray to dock a new item
		sendXMessage(managerCommand, SYSTEM_TRAY_REQUEST_DOCK, gtk_plug_get_id(GTK_PLUG(pluggedIcon)), 0, 0);	
	} else
	{
		gtk_widget_destroy(pluggedIcon);
		return FALSE;
	}
	
	return TRUE;
}




void trayIcon::createTrayIconWindow()
{
	// create the event widget
	eventWidget = gtk_event_box_new();
	gtk_widget_set_events(eventWidget, gtk_widget_get_events(eventWidget) | GDK_BUTTON_PRESS_MASK | GDK_EXPOSURE_MASK);
	gtk_container_add(GTK_CONTAINER(eventWidget), iconWidget);
	
	// create the plug widget
	pluggedIcon = gtk_plug_new(0);
	gtk_container_add(GTK_CONTAINER(pluggedIcon), eventWidget);	
	gtk_plug_construct_for_display(GTK_PLUG(pluggedIcon), gdkDisplay, 0);
	gtk_widget_realize(GTK_WIDGET(pluggedIcon));
	
	gtk_widget_show_all(pluggedIcon);
}



void trayIcon::removeTrayIcon()
{
	
}
	



void trayIcon::sendXMessage(Atom atom, glong message, 
							glong data1, glong data2, glong data3)
{
	XEvent ev;
  
    memset(&ev, 0, sizeof(ev));
    ev.xclient.type = ClientMessage;
    ev.xclient.window = trayWindow;
    ev.xclient.message_type = atom;
    ev.xclient.format = 32;
    ev.xclient.data.l[0] = gdk_x11_get_server_time(pluggedIcon->window);
    ev.xclient.data.l[1] = message;
    ev.xclient.data.l[2] = data1;
    ev.xclient.data.l[3] = data2;
    ev.xclient.data.l[4] = data3;

    gdk_error_trap_push();
    XSendEvent(xdisplay, trayWindow, False, NoEventMask, &ev);
    XSync(xdisplay, False);
    gdk_error_trap_pop();
}



void trayIcon::getSystemTrayWindow()
{
	GdkWindow* wnd;
	XGrabServer(xdisplay);
	
	
	// check if there is an old filter (and remove it)
	if (trayWindow != None)
	{
		wnd = gdk_window_lookup(trayWindow);
		gdk_window_remove_filter(wnd, (GdkFilterFunc)tIsystemTrayChange, this);
	}
	
	
	// get owner that can do a system tray selection
	trayWindow = XGetSelectionOwner(xdisplay, systemTraySelection);
	
	if (trayWindow != None)
	{
		// make manager reporting any change (so icon can be actualized) ...		
		XSelectInput(xdisplay, trayWindow, StructureNotifyMask);
		// ... and install a filter
		wnd = gdk_window_lookup(trayWindow);
		gdk_window_add_filter(wnd, (GdkFilterFunc)tIsystemTrayChange, this);
	}
	
	XUngrabServer(xdisplay);
	XFlush(xdisplay);
}



// -----------------------------------------------------------------------------

GdkFilterReturn tIsystemTrayChange(GdkXEvent *xevent, GdkEvent *event, trayIcon* ti)
{
	XEvent *ev;
	
	ev = (XEvent*) xevent;
	
	if ( (ev->xany.type 			== ClientMessage && 
		  ev->xclient.message_type 	== ti->xManager &&
		  ev->xclient.data.l[1] 	== ti->systemTraySelection ) ||
		 (ev->xany.type				== DestroyNotify &&
		  ev->xany.window			== ti->trayWindow)
		)
		ti -> getSystemTrayWindow();
	
	return GDK_FILTER_CONTINUE;
}
