/***************************************************************************
 *            userutilities.cpp
 *
 *  Mon Feb 14 02:24:32 2005
 *  Copyright  2005  User
 *  Email
 ****************************************************************************/

#include "userutilities.h"


gboolean uu_gnomeIsRunning()
{
	gchar		*path;
	const gchar	*env;
	
	env = g_getenv("DESKTOP_SESSION");
	
	if (env && !strcmp(env,"gnome"))
	{
		path = g_find_program_in_path("gnome-open");
		if (path)
		{
			g_free(path);
			return TRUE;
		}
	}
	
	return FALSE;
}


gboolean uu_openURL(gchar* URL)
{
	gchar			*browserCommand,
					*command,
					*remoteCommand,
					*tmpCommand,
					*fURL;
	gulong			browserMode;
	gboolean 		useDefaults,
					success;
	u_browserInfo	*currentBrowser;
	gint			status;
	
	// get the properties
	settings_getSettings()->getProperties("applications",
										  "useGnomeDefaults", &useDefaults,
										  "browserCommand", &browserCommand,
										  "browserMode", &browserMode,
										  NULL);
	
	remoteCommand = NULL;
	command = NULL;

	fURL = g_strdup_printf("\"%s\"", URL);
	
	if (useDefaults && uu_gnomeIsRunning())
		command = g_strdup_printf("gnome-open \"%s\"", URL);
	else
	{
		currentBrowser = u_getBrowserByCommand(browserCommand);
		
		// a browser was specified
		if (currentBrowser)
		{
			switch(currentBrowser->id)
			{
				case BR_MOZILLA:
					command = g_strdup_printf("%s %s", currentBrowser->command, fURL);
					if (browserMode == WMODE_WINDOW)
						remoteCommand = g_strdup_printf("%s -a firefox -remote \"openURL(%s,new-window)\"", currentBrowser->command, URL);
					else
					if (browserMode == WMODE_TAB)
						remoteCommand = g_strdup_printf("%s -a firefox -remote \"openURL(%s,new-tab)\"", currentBrowser->command, URL);
					else
						remoteCommand = g_strdup_printf("%s -a firefox -remote \"openURL(%s)\"", currentBrowser->command, URL);
					break;
					
				
				case BR_FIREFOX:
					command = g_strdup_printf("%s %s", currentBrowser->command, fURL);
					if (browserMode == WMODE_WINDOW)
						remoteCommand = g_strdup_printf("%s -a firefox -remote \"openURL(%s,new-window)\"", currentBrowser->command, URL);
					else
					if (browserMode == WMODE_TAB)
						remoteCommand = g_strdup_printf("%s -a firefox -remote \"openURL(%s,new-tab)\"", currentBrowser->command, URL);
					else
						remoteCommand = g_strdup_printf("%s -a firefox -remote \"openURL(%s)\"", currentBrowser->command, URL);
					break;

				
				case BR_NETSCAPE:
					command = g_strdup_printf("%s %s", currentBrowser->command, fURL);
					if (browserMode == WMODE_WINDOW)
						remoteCommand = g_strdup_printf("%s -remote \"openURL(%s,new-window)\"", currentBrowser->command, URL);
					else
					if (browserMode == WMODE_CURRENT)
						remoteCommand = g_strdup_printf("%s -remote \"openURL(%s)\"", currentBrowser->command, URL);
					break;					
				
				case BR_OPERA:
					if (browserMode == WMODE_WINDOW)
						command = g_strdup_printf("%s -newwindow %s", currentBrowser->command, fURL);
					else
					if (browserMode == WMODE_TAB)
						command = g_strdup_printf("%s -newpage %s", currentBrowser->command, fURL);
					else
					{
						remoteCommand = g_strdup_printf("%s -remote \"openURL(%s)\"", currentBrowser->command, URL);
						command = g_strdup_printf("%s %s", currentBrowser->command, fURL);
					}
					break;
					
				case BR_EPIPHANY:
				case BR_GALEON:	
					if (browserMode == WMODE_WINDOW)
						command = g_strdup_printf("%s -w %s", currentBrowser->command, fURL);
					else
					if (browserMode == WMODE_TAB)
						command = g_strdup_printf("%s -n %s", currentBrowser->command, fURL);
					else
						command = g_strdup_printf("%s %s", currentBrowser->command, fURL);
					break;
				
				case BR_KONQUEROR:
					command = g_strdup_printf("kfmclient openURL %s", fURL);
					break;
			}
			
		}
		// an own command was defined
		else
			command = g_strdup_printf(browserCommand, fURL);
	}
	
	status = 1;
	success = TRUE;
	
	// try to open the remote command
	if (remoteCommand)
		success = g_spawn_command_line_sync(remoteCommand, NULL, NULL, &status, NULL);
		
	if (status)
		success = g_spawn_command_line_async(command, NULL);

	g_free(fURL);
	if (remoteCommand)
		g_free(remoteCommand);
	g_free(command);
	
	return success;
}


gchar* uu_createGaimDnDMimeType(IMUserDaemon* user, gpointer data)
{
	GString	*str;
	gchar	*proto,
			*convTrail;
	
	str = g_string_new("");
	
	if (user->owner && ((IMOwnerDaemon*)user->owner)->protocol)
	{
		proto = ((IMOwnerDaemon*)user->owner)->protocol->name;
		
		// split the proto
		if (!strcmp(proto, "ICQ / Aim"))
		{
			strtoul(user->info->licqID, &convTrail, 10);
			// the id isn't a number -> the proto is AIM
			if (convTrail && *convTrail)
				proto = "aim";
			else
				proto = "icq";
		}
	}
	else
		proto = "icq";
	
	g_string_append_printf(str, "MIME-Version: 1.0\r\n"
								"Content-Type: application/x-im-contact\r\n"
								"X-IM-Protocol: %s\r\n"
								"X-IM-Username: %s\r\n"
								"X-IM-Alias: %s\r\n"
								"X-IM-Additional: %p\r\n"
								"\r\n",
								proto,
								user->info->licqID,
								user->info->alias,
								data);
	
	return g_string_free(str, FALSE);
}


gboolean uu_getFromGaimDnDMimeType(gchar* str, gchar** id, gchar** alias, IMOwnerDaemon** owner, gpointer* data)
{
	gchar			*startStr, 
					*endStr,
					*valStr;
	
	IMOwnerDaemon	*tmpOwner;
	GList			*search,
					*owners;
					
	
	
	if (!str)
		return FALSE;
	
	if (owner)
	{
		// try to get the protocol
		startStr = g_strstr_len(str, strlen(str), "X-IM-Protocol:");
	
		if (startStr)
		{
			endStr = g_strstr_len(startStr, strlen(startStr), "\r\n");
			if (!endStr)
				endStr = startStr + strlen(startStr);
		
			valStr = (gchar*)g_malloc0(endStr - startStr+1);
			memcpy(valStr, startStr + 14, endStr - startStr - 14);
		
			// remove leading and trailing whitespaces
			valStr = g_strchug(g_strchomp(valStr));
		
			// in Licq ICQ and AIM are one protocol ...
			if (!g_ascii_strncasecmp(valStr, "icq", 3) || !g_ascii_strncasecmp(valStr, "aim", 3))
			{
				g_free(valStr);
				valStr = g_strdup("ICQ / Aim");
			}
		
			// try to find the owner for this protocol name
			owners = IO_getOwnerList();
			for (search = owners; search; search = search->next)
			{
				tmpOwner = (IMOwnerDaemon*)search->data;
				if (!g_ascii_strncasecmp(tmpOwner->protocol->name, valStr, strlen(valStr)))
					break;
			}
		
			if (search)
				*owner = tmpOwner;
		
			g_free(valStr);
		} else
			*owner = NULL;
	}
	
	
	// try to get the id
	
	if (id)
	{
		startStr = g_strstr_len(str, strlen(str), "X-IM-Username:");
	
		if (startStr)
		{
			endStr = g_strstr_len(startStr, strlen(startStr), "\r\n");
			if (!endStr)
				endStr = startStr + strlen(startStr);
		
			*id = (gchar*)g_malloc0(endStr - startStr+1);
			memcpy(*id, startStr + 14, endStr - startStr - 14);
		
			*id = g_strchug(g_strchomp(*id));
		} else
			*id = NULL;
	}

	if (alias)
	{
		// try to get the alias
		startStr = g_strstr_len(str, strlen(str), "X-IM-Alias:");
	
		if (startStr)
		{
			endStr = g_strstr_len(startStr, strlen(startStr), "\r\n");
			if (!endStr)
				endStr = startStr + strlen(startStr);
		
			*alias = (gchar*)g_malloc0(endStr - startStr+1);
			memcpy(*alias, startStr + 11, endStr - startStr - 11);
			*alias = g_strchug(g_strchomp(*alias));
		} else
			*alias = NULL;
	}
	
	if (data)
	{
		// try to get additional data
		startStr = g_strstr_len(str, strlen(str), "X-IM-Additional:");
	
		if (startStr)
		{
			endStr = g_strstr_len(startStr, strlen(startStr), "\r\n");
			if (!endStr)
				endStr = startStr + strlen(startStr);
		
			valStr = (gchar*)g_malloc0(endStr - startStr+1);
			memcpy(valStr, startStr + 16, endStr - startStr - 16);
			valStr = g_strchug(g_strchomp(valStr));
			
			*data = (gpointer)strtoul(valStr, NULL, 16);
			g_free(valStr);
		} else
			*data = NULL;		
	}
	
	
	return TRUE;
}



void uu_getUserFromDnDInfo(gchar* id, gchar* alias, IMOwnerDaemon* owner, IMUserDaemon** targetUser, IMOwnerDaemon** targetOwner)
{
	IMUserDaemon	*dragUser;
	GList			*owners,
					*search;
	
	*targetUser = NULL;
	*targetOwner = NULL;
	
	// nothing to do if we don't have an id
	if (!id)
		return;
		
	// we have an owner
	if (owner)
		dragUser = owner->findUserByID(id);
			
	// no owner? lets see whether we find something
	owners = IO_getOwnerList();
	for (search = owners; search; search = search->next)
		if ( (dragUser = ((IMOwnerDaemon*)search->data)->findUserByID(id)))
		{
			owner = (IMOwnerDaemon*)search->data;
			break;
		}
				
	// still no owner and no user ... no idea
	if (!owner && !dragUser)
		return;

	*targetOwner = owner;
	*targetUser = dragUser;	
}




gchar* uu_getStatusDescription(guint status)
{
	switch(status)
	{
		case BUDDY_STATUS_CONNECT:
			return tr("Connecting...");
		
		case BUDDY_STATUS_OFFLINE:
			 return tr("Offline");
		
		case BUDDY_STATUS_DND:
			 return tr("Do not disturb");
		
		case BUDDY_STATUS_OCC:
			 return tr("Busy");
		
		case BUDDY_STATUS_NA:
			 return tr("Not available");
		
		case BUDDY_STATUS_AWAY:
			 return tr("Away");
		
		case BUDDY_STATUS_FFC:
			 return tr("Free for chat");
		
		case BUDDY_STATUS_INV:
			 return tr("Invisible");
		
		default:
			 return tr("Online");	
	}
}





gchar* uu_getFormattedBuddyName(IMUserInfo* info, gboolean addRealName)
{
	GString *toReturn,
			*realName;
		
	if (info->alias && strcmp(info->alias, "None"))
		toReturn = g_string_new(info->alias);
	else
		toReturn = g_string_new(info->licqID);
	
	if (addRealName)
	{
		realName = g_string_new(info->firstname);
		g_string_append_printf(realName, " %s", info->lastname);
		if (strlen(realName->str)>1)
			g_string_append_printf(toReturn, " (%s)", realName->str);
		g_string_free(realName, TRUE);
	}
	
	
	return g_string_free(toReturn, FALSE);
}



gchar* uu_getRealNameOrAlias(IMUserInfo* info, gboolean useFormatted)
{
	GString	*name;
	gchar	*toReturn;
	
	name = g_string_new(info->firstname);
	if (*name->str)
		name = g_string_append(name, " ");
	name = g_string_append(name, info->lastname);
	
	if (!*name->str)
		name = g_string_append(name, info->alias);
	
	if (useFormatted)
		toReturn = g_markup_escape_text(name->str, strlen(name->str));
	else
		toReturn = g_strdup(name->str);
	
	g_string_free(name, TRUE);
	
	return toReturn;
}



gchar* uu_getSendErrorMessage(guint result, guint status)
{
	if (status == BUDDY_STATUS_OFFLINE)
		return tr(", because you are not connected to the network. Please change your status to \"Online\" by clicking the status button of the main window or by selecting it in the \"System\" -> \"Status\" menu.");
	else
	switch(result)
	{
		case EVENT_RESULT_FAILED:
			return tr(", because there was an error while communicating with the remote. Please check your connection.");
		case EVENT_RESULT_ERROR:
			return tr(", because there was an unknown error. Please check your connection, the user settings and the remote user that you sent this event.");
		case EVENT_RESULT_TIMEOUT:
			return tr(", because the remote user did not respond and caused a timeout. Please try to send this event via the server.");
		default:
			return tr(", because there was some kind of error :)");
	}	
}



void uu_showSendErrorMessage(gchar* title, gchar* beginning, guint result, guint status)
{
	GString	*toShow;
	
	toShow = g_string_new(beginning);
	toShow = g_string_append(toShow, uu_getSendErrorMessage(result, status));
		
	u_showAlertMessage(title, toShow->str, GTK_STOCK_DIALOG_ERROR);
	
	g_string_free(toShow, TRUE);
}



gboolean uu_hasCapat(gulong capats, userEventType event)
{
	switch (event)
	{
		case UE_MESSAGE:
			return (capats & PP_SEND_MSG);
		case UE_URL:
			return (capats & PP_SEND_URL);
		case UE_FILETRANSFER:
		case UE_FILETRANSFERREQUEST:
			return (capats & PP_SEND_FILE);
		case UE_CHAT:
		case UE_CHATREQUEST:
			return (capats & PP_SEND_CHAT);
		case UE_CONTACTS:
		case UE_CONTACTSREQUEST:
			return (capats & PP_SEND_CONTACT);
		case UE_AUTHORIZATION:
		case UE_AUTHORIZATIONREFUSE:
			return (capats & PP_SEND_AUTH);
		case UE_AUTHORIZATIONREQUEST:
			return (capats & PP_SEND_AUTHxREQ);
		case UE_ENCRYPTION:
			return (capats & PP_SEND_SECURE);
		default:
			return TRUE;
	}
}
